@@ -1,7 +1,7 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 
 """
-Copyright (c) 2006-2019 sqlmap developers (http://sqlmap.org/)
+Copyright (c) 2006-2022 sqlmap developers (https://sqlmap.org/)
 See the file 'LICENSE' for copying permission
 """
 
@@ -11,7 +11,7 @@ from lib.core.data import logger
 from lib.core.enums import OS
 from lib.core.exception import SqlmapUndefinedMethod
 
-class Fingerprint:
+class Fingerprint(object):
     """
     This class defines generic fingerprint functionalities for plugins.
     """
