@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 """
-Copyright (c) 2006-2021 sqlmap developers (https://sqlmap.org/)
+Copyright (c) 2006-2022 sqlmap developers (https://sqlmap.org/)
 See the file 'LICENSE' for copying permission
 """
 
@@ -909,6 +909,12 @@ class Connect(object):
 
             socket.setdefaulttimeout(conf.timeout)
 
+        if conf.retryOn and re.search(conf.retryOn, page, re.I):
+            if threadData.retriesCount < conf.retries:
+                warnMsg = "forced retry of the request because of undesired page content"
+                logger.warn(warnMsg)
+                return Connect._retryProxy(**kwargs)
+
         processResponse(page, responseHeaders, code, status)
 
         if not skipLogTraffic:
